!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                                                                   !!
!!   Copyright 2012 David Alonso                                     !!
!!                                                                   !!
!!                                                                   !!
!! This file is part of CRIME.                                       !!
!!                                                                   !!
!! CRIME is free software: you can redistribute it and!or modify it  !!
!! under the terms of the GNU General Public License as published by !!
!! the Free Software Foundation, either version 3 of the License, or !!
!! (at your option) any later version.                               !!
!!                                                                   !!
!! CRIME is distributed in the hope that it will be useful, but      !!
!! WITHOUT ANY WARRANTY; without even the implied warranty of        !!
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU !!
!! General Public License for more details.                          !!
!!                                                                   !!
!! You should have received a copy of the GNU General Public License !!
!! along with CRIME.  If not, see <http://www.gnu.org/licenses/>.    !!
!!                                                                   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
MODULE angles
  USE params_type
  USE math
  USE rngmod

  implicit none

  REAL(FPP), allocatable, dimension(:) :: sqrt_cls_arr

CONTAINS

  SUBROUTINE set_angles(pars)
    implicit none
    type(Params4G), intent(in) :: pars
    INTEGER(4) :: l, stat_alloc

    ALLOCATE(sqrt_cls_arr(0:pars%lmax),STAT=stat_alloc)
    CALL assert_alloc(stat_alloc,"set_angles","sqrt_cls_arr")

    DO l=0,pars%lmin
       sqrt_cls_arr(l)=pars%amp*(REAL(pars%lref)/pars%lmin)**pars%beta
       sqrt_cls_arr(l)=sqrt(sqrt_cls_arr(l))
    END DO
    DO l=pars%lmin+1,pars%lmax
       sqrt_cls_arr(l)=pars%amp*(REAL(pars%lref)/l)**pars%beta
       sqrt_cls_arr(l)=sqrt(sqrt_cls_arr(l))
    END DO
  END SUBROUTINE set_angles

  SUBROUTINE end_angles()
    implicit none
    
    DEALLOCATE(sqrt_cls_arr)
  END SUBROUTINE end_angles

  SUBROUTINE get_alms(rng,lmax,scale,alm_arr)
    implicit none
    INTEGER(4), intent(in) :: lmax
    type(planck_rng), intent(inout) :: rng
    REAL(8), intent(in) :: scale
    COMPLEX(FPP), dimension(:,0:,0:), intent(out) :: alm_arr
    INTEGER(4) :: ll, mm
    REAL(8) :: rms,sqrt_scale
    
    IF(scale<=0) THEN
       WRITE(*,*) "get_alms: Zero variance"
       alm_arr=0.0
    ELSE
       sqrt_scale=sqrt(scale)
       DO ll=0,lmax
          rms=SQRT1HALF*sqrt_cls_arr(ll)*sqrt_scale
          DO mm=0,ll
             alm_arr(1,ll,mm)=rms*CMPLX(rand_gauss(rng),rand_gauss(rng))
          END DO
       END DO
    END IF

  END SUBROUTINE get_alms

END MODULE angles
