!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                                                                   !!
!!   Copyright 2012 David Alonso                                     !!
!!                                                                   !!
!!                                                                   !!
!! This file is part of CRIME.                                       !!
!!                                                                   !!
!! CRIME is free software: you can redistribute it and!or modify it  !!
!! under the terms of the GNU General Public License as published by !!
!! the Free Software Foundation, either version 3 of the License, or !!
!! (at your option) any later version.                               !!
!!                                                                   !!
!! CRIME is distributed in the hope that it will be useful, but      !!
!! WITHOUT ANY WARRANTY; without even the implied warranty of        !!
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU !!
!! General Public License for more details.                          !!
!!                                                                   !!
!! You should have received a copy of the GNU General Public License !!
!! along with CRIME.  If not, see <http://www.gnu.org/licenses/>.    !!
!!                                                                   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
MODULE frequencies
  USE math
  USE params_type

  implicit none
  INTEGER(4), PARAMETER :: N_WORK=10000

CONTAINS

  SUBROUTINE set_numatrix(pars,matrix)
    implicit none
    type(Params4G), intent(in) :: pars
    REAL(8), dimension(:,:), intent(out) :: matrix
    INTEGER(4) :: ii,jj
    REAL(8) :: nu1,nu2
    
    DO ii=1,pars%n_nu
       nu1=pars%nutable(ii,1)
       DO jj=1,pars%n_nu
          nu2=pars%nutable(jj,1)
          matrix(ii,jj)=(nu1*nu2/(pars%nu_ref**2))**(-pars%alpha)*&
               exp(-0.5*(log(nu1/nu2)/pars%xi)**2)
       END DO
    END DO
  END SUBROUTINE set_numatrix

  SUBROUTINE decorrelate_frequencies(pars,eigenvectors,eigenvalues)
    implicit none
    type(Params4G), intent(in) :: pars
    REAL(8), dimension(:,:), intent(out) :: eigenvectors
    REAL(8), dimension(:), intent(out) :: eigenvalues
    REAL(8), dimension(N_WORK) :: work
    INTEGER(4) :: lwork, info

    CALL set_numatrix(pars,eigenvectors)

    lwork=-1
    CALL DSYEV('Vectors','Upper',pars%n_nu,eigenvectors,pars%n_nu,eigenvalues,work,lwork,info)
    lwork=MIN(N_WORK,INT(work(1)))
    WRITE(*,*) "decorrelate_frequencies: calculating eigenvalues, lwork= ", lwork
    CALL DSYEV('Vectors','Upper',pars%n_nu,eigenvectors,pars%n_nu,eigenvalues,work,lwork,info)
    IF(info.ne.0) THEN
       WRITE(*,*) "decorrelate_frequencies: couldn't diagonalize correlation matrix"
    END IF
  END SUBROUTINE decorrelate_frequencies

END MODULE frequencies
