!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                                                                   !!
!!   Copyright 2012 David Alonso                                     !!
!!                                                                   !!
!!                                                                   !!
!! This file is part of CRIME.                                       !!
!!                                                                   !!
!! CRIME is free software: you can redistribute it and!or modify it  !!
!! under the terms of the GNU General Public License as published by !!
!! the Free Software Foundation, either version 3 of the License, or !!
!! (at your option) any later version.                               !!
!!                                                                   !!
!! CRIME is distributed in the hope that it will be useful, but      !!
!! WITHOUT ANY WARRANTY; without even the implied warranty of        !!
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU !!
!! General Public License for more details.                          !!
!!                                                                   !!
!! You should have received a copy of the GNU General Public License !!
!! along with CRIME.  If not, see <http://www.gnu.org/licenses/>.    !!
!!                                                                   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
MODULE io4G

  USE math
  USE params_type
  USE misc_utils
  USE fitstools

  implicit none

CONTAINS

  SUBROUTINE write_map(fname,skymap,header)
    implicit none
    CHARACTER(len=*), intent(in) :: fname
    REAL(FPP), dimension(:,:), intent(in) :: skymap
    CHARACTER(len=80), dimension(:), intent(in) :: header
    
    CALL output_map(skymap,header,fname)
  END SUBROUTINE write_map

  SUBROUTINE set_cl_model(pars)
    implicit none
    type(Params4G), intent(inout) :: pars

    !The Cl model is C_l(nu1,nu2)=[amp*(lref/l)**beta]*[(nu_ref**2/(nu1*nu2))**alpha*exp(-log(nu1/nu2)**2/(2*xi**2))]
    SELECT CASE(pars%cl_model)
    CASE('galactic_synchrotron')
       pars%do_galaxy=1
!       pars%amp=700.0 !This is in Santos, Cooray & Knox
       pars%amp=1600.0 !I think this fits better
       pars%alpha=2.80
!       pars%beta=2.40 !This is in Santos, Cooray & Knox
       pars%beta=3.30 !I think this fits better
       pars%xi=4.0
       pars%nu_ref=130.0
       pars%lref=1000
    CASE('galactic_freefree')
       pars%do_galaxy=0
       pars%amp=0.088
       pars%alpha=2.15
       pars%beta=3.00
       pars%xi=35.0
       pars%nu_ref=130.0
       pars%lref=1000
    CASE('extragalactic_freefree')
       pars%do_galaxy=0
       pars%amp=0.014
       pars%alpha=2.10
       pars%beta=1.00
       pars%xi=35.0
       pars%nu_ref=130.0
       pars%lref=1000
    CASE('point_sources')
       pars%do_galaxy=0
       pars%amp=57.0
       pars%alpha=2.07
       pars%beta=1.10
       pars%xi=1.0
       pars%nu_ref=130.0
       pars%lref=1000
    CASE('custom')
       pars%do_galaxy=0
    CASE default
       WRITE(*,*) "set_cl_model: unknown Cl model ",pars%cl_model
       CALL exit(1)
    END SELECT
    
    IF(pars%do_galaxy==0) THEN
       pars%do_polarization=0
    END IF
    IF(pars%do_polarization.ne.1) pars%do_polarization=0

    WRITE(*,*) "cl_model = ",TRIM(pars%cl_model)
    WRITE(*,*) "amp = ", pars%amp
    WRITE(*,*) "beta = ", pars%beta
    WRITE(*,*) "alpha = ", pars%alpha
    WRITE(*,*) "xi = ", pars%xi
    WRITE(*,*) "lref = ", pars%lref
    WRITE(*,*) "nu_ref = ", pars%nu_ref

    IF(pars%do_polarization==1) THEN
       WRITE(*,*) "xi_polarization = ", pars%xi_polarization
       WRITE(*,*) "beta_polarization = ", pars%beta_polarization
    END IF
  END SUBROUTINE set_cl_model

  SUBROUTINE read_input_params_ForGet(fname_input,pars)
    implicit none
    CHARACTER(len=*), intent(in) :: fname_input
    type(Params4G), intent(out) :: pars
    CHARACTER(len=100) :: buffer, label
    CHARACTER(len=256) :: fname_test
    INTEGER(4) :: ios = 0
    INTEGER(4) :: line = 0
    INTEGER(4) :: pos
    
    CALL set_default_params_ForGet(pars)

    pars%fname_input=fname_input

    OPEN(UNIT=15,FILE=fname_input,STATUS='OLD',ACTION='READ')
    DO WHILE(ios==0)
       READ(15,'(A)',iostat=ios) buffer
       IF(ios==0) THEN
          line=line+1

          IF((buffer(1:1)=='#').or.(buffer(1:1)==' ')) CYCLE
          pos=SCAN(buffer, '=')
          label=buffer(1:pos-1)
          buffer=buffer(pos+1:)

          SELECT CASE(label)
          CASE ('fname_nutable')
             READ(buffer,*,iostat=ios) pars%fname_nutable
             WRITE(*,*) 'fname_nutable = ',TRIM(pars%fname_nutable)
          CASE ('fname_haslam')
             READ(buffer,*,iostat=ios) pars%fname_haslam
             WRITE(*,*) 'fname_haslam = ',TRIM(pars%fname_haslam)
          CASE ('fname_specin')
             READ(buffer,*,iostat=ios) pars%fname_specin
             WRITE(*,*) 'fname_specin = ',TRIM(pars%fname_specin)
          CASE ('prefix_out')
             READ(buffer,*,iostat=ios) pars%prefix_out
             WRITE(*,*) 'prefix_out = ',TRIM(pars%prefix_out)
          CASE ('lmin')
             READ(buffer,*,iostat=ios) pars%lmin
             WRITE(*,*) 'lmin = ',pars%lmin
          CASE ('lmax')
             READ(buffer,*,iostat=ios) pars%lmax
             WRITE(*,*) 'lmax = ',pars%lmax
          CASE ('nside')
             READ(buffer,*,iostat=ios) pars%nside
             WRITE(*,*) 'nside = ',pars%nside
          CASE ('seed')
             READ(buffer,*,iostat=ios) pars%seed
             WRITE(*,*) 'seed = ',pars%seed
          CASE ('cl_model')
             READ(buffer,*,iostat=ios) pars%cl_model
          CASE ('amp')
             READ(buffer,*,iostat=ios) pars%amp
          CASE ('beta')
             READ(buffer,*,iostat=ios) pars%beta
          CASE ('alpha')
             READ(buffer,*,iostat=ios) pars%alpha
          CASE ('xi')
             READ(buffer,*,iostat=ios) pars%xi
          CASE ('lref')
             READ(buffer,*,iostat=ios) pars%lref
          CASE ('nu_ref')
             READ(buffer,*,iostat=ios) pars%nu_ref
          CASE ('do_polarization')
             READ(buffer,*,iostat=ios) pars%do_polarization
          CASE ('xi_polarization')
             READ(buffer,*,iostat=ios) pars%xi_polarization
          CASE ('beta_polarization')
             READ(buffer,*,iostat=ios) pars%beta_polarization
          CASE ('fname_faraday')
             READ(buffer,*,iostat=ios) pars%fname_faraday
             WRITE(*,*) 'fname_faraday = ',TRIM(pars%fname_faraday)
          CASE default
             WRITE(*,*) "read_input_params_ForGet: unknown param ",TRIM(label)," on line ",line
          END SELECT
       END IF
    END DO

    CALL set_cl_model(pars)
    WRITE(buffer,'(I3.3)') 1
    fname_test=TRIM(pars%prefix_out)//"_"//TRIM(buffer)//".fits"
    CALL assert_present(pars%fname_haslam)
    CALL assert_present(pars%fname_specin)
    IF(pars%do_polarization==1) THEN
       CALL assert_present(pars%fname_faraday)
    END IF
    CALL assert_not_present(fname_test)
    
    CALL assert_present(pars%fname_nutable)
    CALL read_nutable(pars%nutable,pars%n_nu,pars%fname_nutable)

  END SUBROUTINE read_input_params_ForGet

  SUBROUTINE read_input_params_JoinT(fname_input,pars)
    implicit none
    CHARACTER(len=*), intent(in) :: fname_input
    type(ParamsJoinT), intent(out) :: pars
    CHARACTER(len=100) :: buffer, label
    CHARACTER(len=256) :: fname_test
    INTEGER(4) :: ios = 0
    INTEGER(4) :: line = 0
    INTEGER(4) :: pos
    
    CALL set_default_params_JoinT(pars)

    pars%fname_input=fname_input

    OPEN(UNIT=15,FILE=fname_input,STATUS='OLD',ACTION='READ')
    DO WHILE(ios==0)
       READ(15,'(A)',iostat=ios) buffer
       IF(ios==0) THEN
          line=line+1

          IF((buffer(1:1)=='#').or.(buffer(1:1)==' ')) CYCLE
          pos=SCAN(buffer, '=')
          label=buffer(1:pos-1)
          buffer=buffer(pos+1:)

          SELECT CASE(label)
          CASE ('prefix_cosmo')
             READ(buffer,*,iostat=ios) pars%prefix_cosmo
             WRITE(*,*) 'prefix_cosmo = ',TRIM(pars%prefix_cosmo)
          CASE ('prefix_synchrotron')
             READ(buffer,*,iostat=ios) pars%prefix_synchrotron
             WRITE(*,*) 'prefix_synchrotron = ',TRIM(pars%prefix_synchrotron)
          CASE ('prefix_galactic_freefree')
             READ(buffer,*,iostat=ios) pars%prefix_galactic_freefree
             WRITE(*,*) 'prefix_galactic_freefree = ',TRIM(pars%prefix_galactic_freefree)
          CASE ('prefix_extragalactic_freefree')
             READ(buffer,*,iostat=ios) pars%prefix_extragalactic_freefree
             WRITE(*,*) 'prefix_extragalactic_freefree = ',TRIM(pars%prefix_extragalactic_freefree)
          CASE ('prefix_point_sources')
             READ(buffer,*,iostat=ios) pars%prefix_point_sources
             WRITE(*,*) 'prefix_point_sources = ',TRIM(pars%prefix_point_sources)
          CASE ('prefix_catalog')
             READ(buffer,*,iostat=ios) pars%prefix_catalog
             WRITE(*,*) 'prefix_catalog = ',TRIM(pars%prefix_catalog)
          CASE ('fname_nutable')
             READ(buffer,*,iostat=ios) pars%fname_nutable
             WRITE(*,*) 'fname_nutable = ',TRIM(pars%fname_nutable)
          CASE ('seed')
             READ(buffer,*,iostat=ios) pars%seed
             WRITE(*,*) 'seed = ',pars%seed
          CASE ('nside')
             READ(buffer,*,iostat=ios) pars%nside
             WRITE(*,*) 'nside = ',pars%nside
          CASE ('do_polarization')
             READ(buffer,*,iostat=ios) pars%do_polarization
          CASE ('polarization_leakage')
             READ(buffer,*,iostat=ios) pars%polarization_leakage
          CASE ('beam_fwhm')
             READ(buffer,*,iostat=ios) pars%beam_fwhm
          CASE ('t_system')
             READ(buffer,*,iostat=ios) pars%tsys
          CASE ('time_integration')
             READ(buffer,*,iostat=ios) pars%time_int
          CASE default
             WRITE(*,*) "read_input_params_JoinT: unknown param ",TRIM(label)," on line ",line
          END SELECT
       END IF
    END DO

    IF(pars%do_polarization.ne.1) pars%do_polarization=0
    IF((pars%polarization_leakage<0.0).or.(pars%polarization_leakage>1.0)) THEN
       WRITE(*,*) "read_input_params_GetGat: wrong polarization leakage: ", pars%polarization_leakage
    ELSE
       WRITE(*,*) "polarization leakage (%) : ", 100*pars%polarization_leakage
    END IF
    WRITE(*,*) "seed : ", pars%seed
    WRITE(*,*) "beam_FWHM (deg) : ", pars%beam_fwhm
    WRITE(*,*) "System temperature (K) : ", 0.001*pars%tsys
    WRITE(*,*) "Integration time (h) : ",pars%time_int

    CALL assert_present(pars%fname_nutable)
    CALL read_nutable(pars%nutable,pars%n_nu,pars%fname_nutable)

    IF((pars%n_nu<=0).or.(pars%nside<=0)) THEN
       WRITE(*,*) "read_input_params_JoinT: wrong nside or n_nu"
       CALL exit(1)
    END IF

    WRITE(buffer,'(I3.3)') 1
    fname_test=TRIM(pars%prefix_cosmo)//"_"//TRIM(buffer)//".fits"
    CALL assert_present(fname_test)
    fname_test=TRIM(pars%prefix_synchrotron)//"_"//TRIM(buffer)//".fits"
    CALL assert_present(fname_test)
    fname_test=TRIM(pars%prefix_galactic_freefree)//"_"//TRIM(buffer)//".fits"
    CALL assert_present(fname_test)
    fname_test=TRIM(pars%prefix_extragalactic_freefree)//"_"//TRIM(buffer)//".fits"
    CALL assert_present(fname_test)
    fname_test=TRIM(pars%prefix_point_sources)//"_"//TRIM(buffer)//".fits"
    CALL assert_present(fname_test)
    fname_test=TRIM(pars%prefix_catalog)//"_"//TRIM(buffer)//".fits"
    CALL assert_not_present(fname_test)

  END SUBROUTINE read_input_params_JoinT
  
END MODULE io4G
