!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                                                                   !!
!!   Copyright 2012 David Alonso                                     !!
!!                                                                   !!
!!                                                                   !!
!! This file is part of CRIME.                                       !!
!!                                                                   !!
!! CRIME is free software: you can redistribute it and!or modify it  !!
!! under the terms of the GNU General Public License as published by !!
!! the Free Software Foundation, either version 3 of the License, or !!
!! (at your option) any later version.                               !!
!!                                                                   !!
!! CRIME is distributed in the hope that it will be useful, but      !!
!! WITHOUT ANY WARRANTY; without even the implied warranty of        !!
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU !!
!! General Public License for more details.                          !!
!!                                                                   !!
!! You should have received a copy of the GNU General Public License !!
!! along with CRIME.  If not, see <http://www.gnu.org/licenses/>.    !!
!!                                                                   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
MODULE params_type

  USE misc_utils

  TYPE Params4G
     CHARACTER(len=256) :: fname_input
     CHARACTER(len=256) :: fname_nutable
     CHARACTER(len=256) :: fname_haslam
     CHARACTER(len=256) :: fname_specin
     CHARACTER(len=256) :: prefix_out
     CHARACTER(len=256) :: cl_model
     INTEGER(4) :: lmin
     INTEGER(4) :: lmax
     INTEGER(4) :: nside
     INTEGER(4) :: seed
     INTEGER(4) :: do_galaxy
     REAL(8) :: amp
     REAL(8) :: beta
     REAL(8) :: alpha
     REAL(8) :: xi
     REAL(8) :: nu_ref
     INTEGER(4) :: lref
     INTEGER(4) :: n_nu
     REAL(8), allocatable, dimension(:,:) :: nutable
     INTEGER(4) :: do_polarization
     REAL(8) :: xi_polarization
     REAL(8) :: beta_polarization
     CHARACTER(len=256) :: fname_faraday
  END TYPE Params4G

  TYPE ParamsJoinT
     CHARACTER(len=256) :: fname_input
     CHARACTER(len=256) :: prefix_cosmo
     CHARACTER(len=256) :: prefix_synchrotron
     CHARACTER(len=256) :: prefix_galactic_freefree
     CHARACTER(len=256) :: prefix_extragalactic_freefree
     CHARACTER(len=256) :: prefix_point_sources
     CHARACTER(len=256) :: prefix_catalog
     CHARACTER(len=256) :: fname_nutable
     INTEGER(4) :: seed
     INTEGER(4) :: n_nu
     REAL(8), allocatable, dimension(:,:) :: nutable
     INTEGER(4) :: nside
     INTEGER(4) :: do_polarization
     REAL(8) :: polarization_leakage
     REAL(8) :: beam_fwhm
     REAL(8) :: tsys
     REAL(8) :: time_int
  END TYPE ParamsJoinT

CONTAINS

  INTEGER(4) FUNCTION linecount(fname)
    implicit none
    CHARACTER(len=*), intent(in) :: fname
    CHARACTER(len=256) :: buffer
    INTEGER(4) :: nlines, i_eos

    i_eos=0
    nlines=0
    CALL assert_present(fname)
    OPEN(UNIT=15,FILE=fname,STATUS='OLD',ACTION='READ')
    DO WHILE(i_eos==0)
       READ(15,'(A)',iostat=i_eos) buffer
       IF(i_eos==0) nlines=nlines+1
    END DO
    CLOSE(UNIT=15)

    linecount=nlines
  END FUNCTION linecount

  SUBROUTINE read_nutable(nutable,n_nu,fname)
    implicit none
    INTEGER(4), intent(out) :: n_nu
    REAL(8), allocatable, dimension(:,:) :: nutable
    CHARACTER(len=*), intent(in) :: fname
    INTEGER(4) :: ii,stat_alloc
    REAL(8), dimension(5) :: dum

    n_nu=linecount(fname)

    ALLOCATE(nutable(1:n_nu,1:3),STAT=stat_alloc)
    CALL assert_alloc(stat_alloc,"read_nutable","nutable")

    WRITE(*,*) "read_nutable: ",n_nu,"nu-values"
    OPEN(UNIT=15,FILE=fname,STATUS='OLD',ACTION='READ')
    DO ii=1,n_nu
       READ(15,*) dum
       nutable(ii,1)=0.5*(dum(3)+dum(2))
       nutable(ii,2)=dum(2)
       nutable(ii,3)=dum(3)
    END DO
    CLOSE(UNIT=15)
  END SUBROUTINE read_nutable

  SUBROUTINE set_default_params_ForGet(pars)
    implicit none
    type(Params4G), intent(out) :: pars

    pars%fname_input="default"
    pars%fname_nutable="default"
    pars%fname_haslam="default"
    pars%fname_specin="default"
    pars%prefix_out="default"
    pars%lmin=10
    pars%lmax=1000
    pars%nside=512
    pars%seed=1234
    pars%n_nu=-1
    pars%do_galaxy=0
    pars%amp=57.0
    pars%beta=1.1
    pars%alpha=2.07
    pars%xi=1.0
    pars%nu_ref=130.0
    pars%lref=1000
    pars%do_polarization=0
    pars%xi_polarization=3.0
    pars%beta_polarization=2.5
    pars%fname_faraday="default"
  END SUBROUTINE set_default_params_ForGet

  SUBROUTINE set_default_params_JoinT(pars)
    implicit none
    type(ParamsJoinT), intent(out) :: pars
    
    pars%fname_input="default"
    pars%prefix_cosmo="default"
    pars%prefix_synchrotron="default"
    pars%prefix_galactic_freefree="default"
    pars%prefix_extragalactic_freefree="default"
    pars%prefix_point_sources="default"
    pars%prefix_catalog="default"
    pars%fname_nutable="default"
    pars%seed=1234
    pars%n_nu=-1
    pars%nside=-1
    pars%do_polarization=0
    pars%polarization_leakage=-1.0
    pars%beam_fwhm=-1.0
    pars%tsys=-1.0
    pars%time_int=-1.0
  END SUBROUTINE set_default_params_JoinT

  SUBROUTINE free_params_ForGet(pars)
    implicit none
    type(Params4G), intent(inout) :: pars

    DEALLOCATE(pars%nutable)
  END SUBROUTINE free_params_ForGet

  SUBROUTINE free_params_JoinT(pars)
    implicit none
    type(ParamsJoinT), intent(inout) :: pars

    DEALLOCATE(pars%nutable)
  END SUBROUTINE free_params_JoinT

END MODULE params_type
