///////////////////////////////////////////////////////////////////////
//                                                                   //
//   Copyright 2012 David Alonso                                     //
//                                                                   //
//                                                                   //
// This file is part of GetHI.                                       //
//                                                                   //
// GetHI is free software: you can redistribute it and/or modify it  //
// under the terms of the GNU General Public License as published by //
// the Free Software Foundation, either version 3 of the License, or //
// (at your option) any later version.                               //
//                                                                   //
// GetHI is distributed in the hope that it will be useful, but      //
// WITHOUT ANY WARRANTY; without even the implied warranty of        //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU //
// General Public License for more details.                          //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with GetHI.  If not, see <http://www.gnu.org/licenses/>.    //
//                                                                   //
///////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gsl/gsl_rng.h>
#include "common.h"

#ifdef _HAVE_OMP
#include <omp.h>
static double relbeg,relend,absbeg,absend;
#else //_HAVE_OMP
#include <time.h>
static time_t relbeg,relend,absbeg,absend;
#endif //_HAVE_OMP

int signed_seed_rng=1234;
unsigned int seed_rng=1234;
int n_grid=32;
double l_box=100.;
double nu_min;
double nu_max;
int n_nu;
long n_side;
char fnamePk[256]="none";
char fnameOut[256]="none";
double pos_obs[3]={0.,0.,0.};
int do_psources=0;

void error_mem_out(void)
{
  //////
  // Memory shortage handler
  fprintf(stderr,"GetHI: Out of memory \n");
  exit(1);
}

void error_open_file(char *fname)
{
  fprintf(stderr,"GetHI: Couldn't open file %s \n",fname);
  exit(1);
}

void error_read_line(char *fname,int nlin)
{
  fprintf(stderr,"GetHI: Error reading file %s, line %d \n",fname,nlin);
  exit(1);
}

int linecount(FILE *f)
{
  //////
  // Counts #lines from file
  int i0=0;
  char ch[1000];
  while((fgets(ch,sizeof(ch),f))!=NULL) {
    i0++;
  }
  return i0;
}

void timer(int i)
{
  /////
  // Timing routine
  // timer(0) -> initialize relative clock
  // timer(1) -> read relative clock
  // timer(2) -> read relative clock and initialize it afterwards
  // timer(4) -> initialize absolute clock
  // timer(5) -> read absolute clock
#ifdef _HAVE_OMP
  if(i==0)
    relbeg=omp_get_wtime();
  else if(i==1) {
    relend=omp_get_wtime();
    printf("    Relative time ellapsed %.1lf ms\n",1000*(relend-relbeg));
  }
  else if(i==2) {
    relend=omp_get_wtime();
    printf("    Relative time ellapsed %.1lf ms\n",1000*(relend-relbeg));
    relbeg=omp_get_wtime();
  }
  else if(i==4)
    absbeg=omp_get_wtime();
  else if(i==5) {
    absend=omp_get_wtime();
    printf("    Total time ellapsed %.1lf ms\n",1000*(absend-absbeg));
  }
#else //_HAVE_OMP
  int diff;

  if(i==0)
    relbeg=time(NULL);
  else if(i==1) {
    relend=time(NULL);
    diff=(int)(difftime(relend,relbeg));
    printf("    Relative time ellapsed %02d:%02d:%02d \n",
           diff/3600,(diff/60)%60,diff%60);
  }
  else if(i==2) {
    relend=time(NULL);
    diff=(int)(difftime(relend,relbeg));
    printf("    Relative time ellapsed %02d:%02d:%02d \n",
           diff/3600,(diff/60)%60,diff%60);
    relbeg=time(NULL);
  }
  else if(i==4)
    absbeg=time(NULL);
  else if(i==5) {
    absend=time(NULL);
    diff=(int)(difftime(absend,absbeg));
    printf("    Total time ellapsed %02d:%02d:%02d \n",
           diff/3600,(diff/60)%60,diff%60);
  }
#endif //_HAVE_OMP
}

int wrap_int(int n)
{
  if(n<0)
    return wrap_int(n+n_grid);
  else if(n>=n_grid)
    return wrap_int(n-n_grid);
  else
    return n;
}

flouble wrap_double(flouble x)
{
  if(x<0)
    return wrap_double(x+l_box);
  else if(x>=l_box)
    return wrap_double(x-l_box);
  else
    return x;
}
