///////////////////////////////////////////////////////////////////////
//                                                                   //
//   Copyright 2012 David Alonso                                     //
//                                                                   //
//                                                                   //
// This file is part of GetHI.                                       //
//                                                                   //
// GetHI is free software: you can redistribute it and/or modify it  //
// under the terms of the GNU General Public License as published by //
// the Free Software Foundation, either version 3 of the License, or //
// (at your option) any later version.                               //
//                                                                   //
// GetHI is distributed in the hope that it will be useful, but      //
// WITHOUT ANY WARRANTY; without even the implied warranty of        //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU //
// General Public License for more details.                          //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with GetHI.  If not, see <http://www.gnu.org/licenses/>.    //
//                                                                   //
///////////////////////////////////////////////////////////////////////

#ifndef _DOIC_COMMON_
#define _DOIC_COMMON_

#define NU_21 1420.40575177
#define DYNAMIC_SIZE 1
#define RTOD 57.2957795
#define TWOPIPIINVLOGTEN  0.1166503235296796 //ln(10)/(2*pi^2)
#define TWOPIPIINV  0.05066059182116889 //1/(2*pi^2)
#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
#define ABS(a)   (((a) < 0) ? -(a) : (a)) //Absolute value
#define CLAMP(x, low, high)  (((x) > (high)) ? (high) : (((x) < (low)) ? (low) : (x)))

#ifdef _LONGIDS
typedef long lint;
#else //_LONGIDS
typedef int lint;
#endif //_LONGIDS

#ifdef _SPREC
typedef float flouble;
#else //_SPREC
typedef double flouble;
#endif //_SPREC

//Cosmology
extern char fnamePk[256];
extern double OmegaM;
extern double OmegaL;
extern double OmegaB;
extern double hhub;
extern double weos;
extern double n_scal;
extern double sig8;
//Derived parameters
extern double fgrowth_0;
extern double hubble_0;

//Box parameters
extern double nu_max;
extern double nu_min;
extern double z_max;
extern double z_min;
extern double r_max;
extern double r_min;
extern double l_box;
extern int n_grid;
extern long n_side;
extern int n_nu;

//Output parameters
extern char fnameOut[256];

//RNG seed
extern int signed_seed_rng;
extern unsigned int seed_rng;

//Observer coordinates
extern double pos_obs[3];

//Functionality
extern int do_psources;

/*
typedef struct {
  double cth;
  double phi;
  double dz_rsd;
  double zz;
  double mass_HI;
} Particle;
*/

void error_mem_out(void);

void error_open_file(char *fname);

void error_read_line(char *fname,int nlin);

int linecount(FILE *f);

void timer(int i);

int wrap_int(int n);

flouble wrap_double(flouble x);

//////
// Functions defined in random.c
gsl_rng *init_rng(unsigned int seed);

double rng_01(gsl_rng *rng);

int rng_poisson(double lambda,gsl_rng *rng);

void rng_delta_gauss(double *module,double *phase,
		     gsl_rng *rng,double sigma2);

void end_rng(gsl_rng *rng);


//////
// Functions defined in cosmo.c
double pk_linear0(double lgk);

void cosmo_set(void);

double r_of_z(double z);

double z_of_r(double r);

double dgrowth_of_r(double r);

double vgrowth_of_r(double r);

void end_cosmo(void);

//////
// Functions defined in io.c
void read_run_params(char *fname);

void write_maps(char *fname,flouble *T_maps,flouble *T_maps_psources);

//////
// Functions defined in user_defined.c
double fraction_HI(double z);

double bias_HI(double z);

//////
// Functions defined in fourier.c
void create_d_and_vr_fields(unsigned int *seed,flouble **delta,
			    flouble **radial_velocity);

int *get_point_sources(flouble *delta_grid,unsigned int *seed);

//////
// Functions defined in grid_tools.c
void get_HI(flouble *delta_grid,flouble *v_rad);

//////
// Functions defined in pixelize.c
flouble *mk_T_maps(unsigned int *seed,flouble *mass_grid,flouble *dz_grid);

flouble *mk_psources_maps(unsigned int *seed,int *nsources,flouble *dz_grid);

//////
// Functions defined in psources.c
void setup_psources(void);

double n_of_z_psources(double z);

double bias_psources(double z);

double draw_luminosity(double redshift,gsl_rng *rng);

double draw_specin(gsl_rng *rng);


#endif //_DOIC_COMMON_
